
/**************************************************************************/
/* 
 * RETRO ROCKET LED LAMP WITH THERMOMETER
 * V4HE, 10/02/2019
 * 
 * This is a quick and dirty demo that has been hacked together using:
 * 
 * demo sketch for the Adafruit MCP9808 breakout
 * TM1637 4 digit 7 segment display
 * FastLED Library
 * TM1367+DS18B20 temperature sensor by Floris Wouterlood 
 * 
 * The code reads the MCP9808 and calibrates the reading to compensate for
 * internal case heating (note that what I did won't work over the full
 * range of temperature- you need something more complicated for that!).
 * The display is updated and the LED pixel colour is changed based on
 * the following temperature ranges:
 * 
 * temp >=15.5: Blue
 * 16.0 <= temp <= 20.0: Green
 * 20.5 <= temp <= 24.5: Yellow
 * 25.0 <= temp <= 29.5: Orange
 * temp >= 30: Red
 * 
*/
/**************************************************************************/

#include <TM1637Display.h>
#include <Wire.h>
#include "Adafruit_MCP9808.h"
#include <FastLED.h>

// Create the MCP9808 temperature sensor object
Adafruit_MCP9808 tempsensor = Adafruit_MCP9808();


//TM1637 pins
  const int CLK = 9;                // set the CLK pin connection to the display
  const int DIO = 8;                // set the DIO pin connection to the display

//Neopixel data pin
#define DATA_PIN 6

//Button
#define IN_SW1 2


const uint8_t DEGREES[] = {
   0x0, 0x0,
   SEG_A | SEG_B | SEG_G | SEG_F,    // superscript o
   SEG_A | SEG_F | SEG_E | SEG_D,    // C
   };

  TM1637Display display(CLK, DIO);  // set up the 4-Digit Display

//number of LEDs in the Neopixel ring
#define NUM_LEDS 12

//Define the array of leds
CRGB leds[NUM_LEDS];


void setup() {
    Serial.begin(9600);
  while (!Serial); //waits for serial terminal to be open, necessary in newer arduino boards.
  Serial.println("MCP9808 demo");

//TM1637 setup
  display.setBrightness(0x0a);      // set display to maximum brightness
  
//Neopixel setup
  FastLED.addLeds<NEOPIXEL, DATA_PIN>(leds, NUM_LEDS);

//MCP9808 setup
  // Make sure the sensor is found, you can also pass in a different i2c
  // address with tempsensor.begin(0x19) for example, also can be left in blank for default address use
  // Also there is a table with all addres possible for this sensor, you can connect multiple sensors
  // to the same i2c bus, just configure each sensor with a different address and define multiple objects for that
  //  A2 A1 A0 address
  //  0  0  0   0x18  this is the default address
  //  0  0  1   0x19
  //  0  1  0   0x1A
  //  0  1  1   0x1B
  //  1  0  0   0x1C
  //  1  0  1   0x1D
  //  1  1  0   0x1E
  //  1  1  1   0x1F
  if (!tempsensor.begin(0x18)) {
    Serial.println("Couldn't find MCP9808! Check your connections and verify the address is correct.");
    while (1);
  }
    
   Serial.println("Found MCP9808!");

  tempsensor.setResolution(0); // sets the resolution mode of reading, the modes are defined in the table bellow:
  // Mode Resolution SampleTime
  //  0    0.5°C       30 ms
  //  1    0.25°C      65 ms
  //  2    0.125°C     130 ms
  //  3    0.0625°C    250 ms
}

//Global variables- temperature
float c_sns;  //sensor temp *C
float c_cal;  //calibrated temp
int oldTemp;
int newTemp;

//Button state
bool buttonState = false;

void loop()
{
  Serial.println("wake up MCP9808.... "); // wake up MCP9808 - power consumption ~200 mikro Ampere
  tempsensor.wake();   // wake up, ready to read!

  // Read and print out the temperature, also shows the resolution mode used for reading.
  Serial.print("Resolution in mode: ");
  Serial.println (tempsensor.getResolution());
  c_sns = tempsensor.readTempC();
  Serial.print("Sensor Temp: "); 
  Serial.print(c_sns, 4); Serial.print("*C.");
  c_cal = c_sns - 4.0;  //reduce temperature reading by 4*C to compensate for case heating
  Serial.print("Calibrated temp: "); 
  Serial.print(c_cal, 4); Serial.print("*C.");
  oldTemp = newTemp;
  newTemp = (int)c_cal;

    if(oldTemp != newTemp)
    {
      to_the_display(c_cal);
      updateLEDs(c_cal);
    }
    else {
      oldTemp = newTemp;
    } 
  
  delay(10000);
  Serial.println("Shutdown MCP9808.... ");
  tempsensor.shutdown_wake(1); // shutdown MSP9808 - power consumption ~0.1 mikro Ampere, stops temperature sampling
  Serial.println("");
  delay(200);
}

// subroutines =======================================================================
void to_the_display (float temp_c)
{
  display.setSegments(DEGREES); //Display the Variable value;
  display.showNumberDec(temp_c,false,2,0);
}


void updateLEDs(float temp_c)
{
  if(temp_c <= 15.5)  //BLUE
  {
    for(int dot = 0; dot < NUM_LEDS; dot++)
    {
      leds[dot] = CRGB::Blue;
      FastLED.show();
      delay(100);
    }
     Serial.println("LEDs are Blue");
  }
  else if(temp_c >= 16.0 && temp_c <= 20.0) //GREEN
  {
    for(int dot = 0; dot < NUM_LEDS; dot++)
    {
      leds[dot] = CRGB::Green;
      FastLED.show();
      delay(50);
    }
    Serial.println("LEDs are Green");
  }
  else if(temp_c >= 20.5 && temp_c <= 24.5) //YELLOW
  { 
    for(int dot = 0; dot < NUM_LEDS; dot++)
    {
      leds[dot] = CRGB::Yellow;
      FastLED.show();
      delay(50);
    }
    Serial.println("LEDs are Yellow");
  }
  else if(temp_c >= 25.0 && temp_c <= 29.5) //ORANGE
  {
    for(int dot = 0; dot < NUM_LEDS; dot++)
    {
      leds[dot] = CRGB::DarkOrange;
      FastLED.show();
      delay(50);
    }
    Serial.println("LEDs are Orange");
  }
  else  // >30 --> //RED
  {
    for(int dot = 0; dot < NUM_LEDS; dot++)
    {
      leds[dot] = CRGB::Red;
      FastLED.show();
      delay(50);
    }
    Serial.println("LEDs are Red");
  }
}
